@@ -69,7 +69,8 @@ def create_credential_resolver(session, cache=None):
     instance_metadata_provider = InstanceMetadataProvider(
         iam_role_fetcher=InstanceMetadataFetcher(
             timeout=metadata_timeout,
-            num_attempts=num_attempts)
+            num_attempts=num_attempts,
+            user_agent=session.user_agent())
     )
     assume_role_provider = AssumeRoleProvider(
         load_config=lambda: session.full_config,
@@ -97,10 +98,7 @@ def create_credential_resolver(session, cache=None):
         container_provider,
         instance_metadata_provider
     ]
-
-    explicit_profile = session.get_config_variable('profile',
-                                                   methods=('instance',))
-    if explicit_profile is not None:
+    if session.instance_variables().get('profile') is not None:
         # An explicitly provided profile will negate an EnvProvider.
         # We will defer to providers that understand the "profile"
         # concept to retrieve credentials.
@@ -444,6 +442,8 @@ class RefreshableCredentials(Credentials):
             # set of temporary credentials we have.
             return
         self._set_from_data(metadata)
+        self._frozen_credentials = ReadOnlyCredentials(
+            self._access_key, self._secret_key, self._token)
         if self._is_expired():
             # We successfully refreshed credentials but for whatever
             # reason, our refreshing function returned credentials
@@ -454,14 +454,25 @@ class RefreshableCredentials(Credentials):
                    "refreshed credentials are still expired.")
             logger.warning(msg)
             raise RuntimeError(msg)
-        self._frozen_credentials = ReadOnlyCredentials(
-            self._access_key, self._secret_key, self._token)
 
     @staticmethod
     def _expiry_datetime(time_str):
         return parse(time_str)
 
     def _set_from_data(self, data):
+        expected_keys = ['access_key', 'secret_key', 'token', 'expiry_time']
+        if not data:
+            missing_keys = expected_keys
+        else:
+            missing_keys = [k for k in expected_keys if k not in data]
+
+        if missing_keys:
+            message = "Credential refresh failed, response did not contain: %s"
+            raise CredentialRetrievalError(
+                provider=self.method,
+                error_msg=message % ', '.join(missing_keys),
+            )
+
         self.access_key = data['access_key']
         self.secret_key = data['secret_key']
         self.token = data['token']
@@ -525,7 +536,7 @@ class DeferredRefreshableCredentials(RefreshableCredentials):
         self._frozen_credentials = None
 
     def refresh_needed(self, refresh_in=None):
-        if any(part is None for part in [self._access_key, self._secret_key]):
+        if self._frozen_credentials is None:
             return True
         return super(DeferredRefreshableCredentials, self).refresh_needed(
             refresh_in
